﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using CodeFirstDemo.Entities;

namespace CodeFirstDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            BasicDemo();

            SqlDemo();

            LinqToEntitiesDemo();
        }

        private static void BasicDemo()
        {
            Console.WriteLine("Add new teams:");
            var newTeam = Console.ReadLine();

            using (var db = new GameDbContext())
            {
                db.Teams.Add(new Team() { Name = newTeam });

                db.SaveChanges();
            }

            Console.WriteLine("Teams:");

            using (var db = new GameDbContext())
            {
                var teams = db.Teams.OrderBy(team => team.Name).ToList();

                foreach (var team in teams)
                {
                    Console.WriteLine(team.Name);
                }
            }
        }

        private static void SqlDemo()
        {
            // raw SQL
            using (var db = new GameDbContext())
            {
                var allTeams = db.Teams.SqlQuery("SELECT * FROM Teams ORDER BY name").ToList();
                Console.WriteLine("Teams:");
                foreach (var team in allTeams)
                {
                    Console.WriteLine(team.Name);
                }
            }

            // Entity SQL
            List<Team> teams;
            var esql = "SELECT VALUE t FROM Teams AS t ORDER BY t.Name";
            using (var objectContext = ((IObjectContextAdapter)new GameDbContext()).ObjectContext)
            {
                var query = objectContext.CreateQuery<Team>(esql);

                teams = query.ToList();
            }

            Console.WriteLine("Teams:");
            foreach (var team in teams)
            {
                Console.WriteLine(team.Name);
            }
        }

        private static void LinqToEntitiesDemo()
        {
            // update
            using (var db = new GameDbContext())
            {
                //db.Database.Log = Console.WriteLine;
                var team = db.Teams.First(t => t.Name == "SuperTeam");
                team.Name = "MegaTeam";
                db.SaveChanges();
            }

            // update disconnected
            Team myTeam;
            using (var db = new GameDbContext())
            {
                myTeam = db.Teams.First(t => t.Name == "MegaTeam");
            }

            myTeam.Name = "SuperTeam";

            using (var db = new GameDbContext())
            {
                //db.Database.Log = Console.WriteLine;
                db.Entry(myTeam).State = EntityState.Modified;
                db.SaveChanges();
            }

            var player = new Player
            {
                Name = "Peter"
            };

            using (var db = new GameDbContext())
            {
                db.Players.Add(player);
                player.Team = new Team { Name = "Petovia" };
                db.SaveChanges();
            }

            // select (navigation properties)
            Console.WriteLine("Eager Loading");
            using (var db = new GameDbContext())
            {
                var team = db.Teams.Include(t => t.Players).First(t => t.Name == "Petovia");
                Console.WriteLine(team.Players.Count);
            }

            Console.WriteLine("Explicit Loading");
            Team team2;
            using (var db = new GameDbContext())
            {
                team2 = db.Teams.First(t => t.Name == "Petovia");
            }

            using (var db = new GameDbContext())
            {
                //db.Entry(team2).State = EntityState.Unchanged;
                db.Teams.Attach(team2);
                db.Entry(team2).Collection(t => t.Players).Load();
                Console.WriteLine(team2.Players.Count);
            }

            // delete (navigation properties)
            using (var db = new GameDbContext())
            {
                var team = db.Teams.Include(t => t.Players).First(t => t.Name == "Petovia");
                db.Teams.Remove(team);
                db.SaveChanges();
            }

            //client side vs server side

            // init some data
            using (var db = new GameDbContext())
            {
                db.Teams.Add(new Team
                {
                    Name = "TestTeam",
                    Country = "Mexico",
                    Players = new List<Player>
                    {
                        new Player {Name = "Martin"},
                        new Player {Name = "Tomas"}
                    }
                });
                db.SaveChanges();
            }

            using (var db = new GameDbContext())
            {
                //db.Database.Log = Console.WriteLine;
                // client side
                Console.WriteLine("Client side:");
                var myTeamClient = db.Teams.First(t => t.Name == "TestTeam");

                var superTeamMembersCount = myTeamClient.Players.Count;
                Console.WriteLine(superTeamMembersCount);

                // server side
                Console.WriteLine("Server side:");
                var myTeamServer = db.Teams.First(t => t.Name == "TestTeam");

                var superTeamMembersCount2 = db.Entry(myTeamServer)
                    .Collection(t => t.Players)
                    .Query()
                    .Count();
                Console.WriteLine(superTeamMembersCount2);
            }
        }
    }
}
